@@ -10,8 +10,10 @@
 #define SESSION_H
 
 #include "config.h"
+#if ENABLE_NLS
 #include <libintl.h>
 #include <locale.h>
+#endif
 
 #include <list>
 #include <string>
@@ -55,6 +57,7 @@ struct tracepoint_derived_probe_group;
 struct hrtimer_derived_probe_group;
 struct procfs_derived_probe_group;
 struct dynprobe_derived_probe_group;
+struct java_derived_probe_group;
 struct embeddedcode;
 struct stapdfa;
 class translator_output;
@@ -322,6 +325,7 @@ public:
   hrtimer_derived_probe_group* hrtimer_derived_probes;
   procfs_derived_probe_group* procfs_derived_probes;
   dynprobe_derived_probe_group* dynprobe_derived_probes;
+  java_derived_probe_group* java_derived_probes;
 
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
@@ -343,6 +347,10 @@ public:
   struct module_cache* module_cache;
   std::vector<std::string> build_ids;
 
+  // Secret benchmarking options
+  unsigned long benchmark_sdt_loops;
+  unsigned long benchmark_sdt_threads;
+
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
 
