@@ -25,6 +25,11 @@ using namespace std;
 
 
 
+visitable::~visitable ()
+{
+}
+
+
 expression::expression ():
   type (pe_unknown), tok (0)
 {
@@ -111,7 +116,7 @@ probe_point::probe_point ():
 unsigned probe::last_probeidx = 0;
 
 probe::probe ():
-  body (0), tok (0), systemtap_v_conditional (0), privileged (false)
+  body (0), base (0), tok (0), systemtap_v_conditional (0), privileged (false)
 {
   this->name = string ("probe_") + lex_cast(last_probeidx ++);
 }
@@ -120,16 +125,17 @@ probe::probe ():
 // Copy constructor, but with overriding probe-point.  To be used when
 // mapping script-level probe points to another one, early during pass
 // 2.  There should be no symbol resolution done yet.
-probe::probe(const probe& p, probe_point* l)
+probe::probe(probe* p, probe_point* l)
 {
+  this->base = p;
   this->name = string ("probe_") + lex_cast(last_probeidx ++);
-  this->tok = p.tok;
+  this->tok = p->tok;
   this->locations.push_back(l);
-  this->body = p.body; // NB: not needed to be copied yet; a later derived_probe will
-  this->privileged = p.privileged;
-  this->systemtap_v_conditional = p.systemtap_v_conditional;
-  assert (p.locals.size() == 0);
-  assert (p.unused_locals.size() == 0);
+  this->body = p->body; // NB: not needed to be copied yet; a later derived_probe will
+  this->privileged = p->privileged;
+  this->systemtap_v_conditional = p->systemtap_v_conditional;
+  assert (p->locals.size() == 0);
+  assert (p->unused_locals.size() == 0);
 }
 
 
@@ -333,6 +339,15 @@ void binary_expression::print (ostream& o) const
     << " (" << *right << ")";
 }
 
+void regex_query::print (ostream& o) const
+{
+  // NB: we need a custom printer, because the parser does not accept
+  // a parenthesized RHS.
+  o << "(" << *left << ") "
+    << op
+    << " " << *re;
+}
+
 
 void unary_expression::print (ostream& o) const
 {
@@ -348,7 +363,7 @@ void array_in::print (ostream& o) const
       operand->indexes[i]->print (o);
     }
   o << "] in ";
-  operand->base->print_indexable (o);
+  operand->base->print (o);
 }
 
 void post_crement::print (ostream& o) const
@@ -499,7 +514,7 @@ void functiondecl::printsig (ostream& o) const
 
 void arrayindex::print (ostream& o) const
 {
-  base->print_indexable (o);
+  base->print (o);
   o << "[";
   for (unsigned i=0; i<indexes.size(); i++)
     o << (i>0 ? ", " : "") << *indexes[i];
@@ -1045,7 +1060,7 @@ void foreach_loop::print (ostream& o) const
 	o << (sort_direction > 0 ? "+" : "-");
     }
   o << "] in ";
-  base->print_indexable (o);
+  base->print (o);
   if (sort_direction != 0 && sort_column == 0)
     {
       switch (sort_aggr)
@@ -1177,14 +1192,41 @@ void probe::printsig (ostream& o) const
     }
 }
 
+const probe* 
+probe::basest () const 
+{
+  return base ? base->basest() : this; 
+}
+
+
+const probe* 
+probe::almost_basest () const {
+  if (base)
+    return base->base ? base->almost_basest() : this; 
+  else
+    return 0;
+}
+
 
 void
-probe::collect_derivation_chain (std::vector<probe*> &probes_list)
+probe::collect_derivation_chain (std::vector<probe*> &probes_list) const
 {
-  probes_list.push_back(this);
+  probes_list.push_back(const_cast<probe*>(this));
+  if (base)
+    base->collect_derivation_chain(probes_list);
 }
 
 
+void
+probe::collect_derivation_pp_chain (std::vector<probe_point*> &pp_list) const
+{
+  pp_list.push_back(const_cast<probe_point*>(this->locations[0]));
+  if (base)
+    base->collect_derivation_pp_chain(pp_list);
+}
+
+
+
 void probe_point::print (ostream& o, bool print_extras) const
 {
   for (unsigned i=0; i<components.size(); i++)
@@ -1518,36 +1560,6 @@ hist_op::visit (visitor *u)
   u->visit_hist_op (this);
 }
 
-void
-indexable::print_indexable (std::ostream& o) const
-{
-  const symbol *sym;
-  const hist_op *hist;
-  classify_const_indexable(this, sym, hist);
-  if (sym)
-    sym->print (o);
-  else
-    {
-      assert (hist);
-      hist->print (o);
-    }
-}
-
-void
-indexable::visit_indexable (visitor* u)
-{
-  symbol *sym;
-  hist_op *hist;
-  classify_indexable(this, sym, hist);
-  if (sym)
-    sym->visit (u);
-  else
-    {
-      assert (hist);
-      hist->visit (u);
-    }
-}
-
 
 bool
 indexable::is_symbol(symbol *& sym_out)
@@ -1563,20 +1575,6 @@ indexable::is_hist_op(hist_op *& hist_out)
   return false;
 }
 
-bool
-indexable::is_const_symbol(const symbol *& sym_out) const
-{
-  sym_out = NULL;
-  return false;
-}
-
-bool
-indexable::is_const_hist_op(const hist_op *& hist_out) const
-{
-  hist_out = NULL;
-  return false;
-}
-
 bool
 symbol::is_symbol(symbol *& sym_out)
 {
@@ -1584,19 +1582,6 @@ symbol::is_symbol(symbol *& sym_out)
   return true;
 }
 
-bool
-symbol::is_const_symbol(const symbol *& sym_out) const
-{
-  sym_out = this;
-  return true;
-}
-
-const token *
-symbol::get_tok() const
-{
-  return tok;
-}
-
 bool
 hist_op::is_hist_op(hist_op *& hist_out)
 {
@@ -1604,19 +1589,6 @@ hist_op::is_hist_op(hist_op *& hist_out)
   return true;
 }
 
-bool
-hist_op::is_const_hist_op(const hist_op *& hist_out) const
-{
-  hist_out = this;
-  return true;
-}
-
-const token *
-hist_op::get_tok() const
-{
-  return tok;
-}
-
 void
 classify_indexable(indexable* ix,
 		   symbol *& array_out,
@@ -1626,21 +1598,11 @@ classify_indexable(indexable* ix,
   hist_out = NULL;
   assert(ix != NULL);
   if (!(ix->is_symbol (array_out) || ix->is_hist_op (hist_out)))
-    throw semantic_error(_("Expecting symbol or histogram operator"), ix->get_tok());
+    throw semantic_error(_("Expecting symbol or histogram operator"), ix->tok);
   if (!(hist_out || array_out))
-    throw semantic_error(_("Failed to classify indexable"), ix->get_tok());
+    throw semantic_error(_("Failed to classify indexable"), ix->tok);
 }
 
-void
-classify_const_indexable(const indexable* ix,
-			 const symbol *& array_out,
-			 const hist_op *& hist_out)
-{
-  array_out = NULL;
-  hist_out = NULL;
-  if (!(ix->is_const_symbol(array_out) || ix->is_const_hist_op(hist_out)))
-    throw semantic_error(_("Expecting symbol or histogram operator"), ix->get_tok());
-}
 
 // ------------------------------------------------------------------------
 
@@ -1727,13 +1689,7 @@ traversing_visitor::visit_for_loop (for_loop* s)
 void
 traversing_visitor::visit_foreach_loop (foreach_loop* s)
 {
-  symbol *array = NULL;
-  hist_op *hist = NULL;
-  classify_indexable (s->base, array, hist);
-  if (array)
-    array->visit(this);
-  else
-    hist->visit(this);
+  s->base->visit(this);
 
   for (unsigned i=0; i<s->indexes.size(); i++)
     s->indexes[i]->visit (this);
@@ -1915,13 +1871,7 @@ traversing_visitor::visit_arrayindex (arrayindex* e)
   for (unsigned i=0; i<e->indexes.size(); i++)
     e->indexes[i]->visit (this);
 
-  symbol *array = NULL;
-  hist_op *hist = NULL;
-  classify_indexable(e->base, array, hist);
-  if (array)
-    return array->visit(this);
-  else
-    return hist->visit(this);
+  e->base->visit(this);
 }
 
 void
@@ -2212,19 +2162,11 @@ varuse_collecting_visitor::visit_arrayindex (arrayindex *e)
   symbol *array = NULL;
   hist_op *hist = NULL;
   classify_indexable(e->base, array, hist);
+  expression *value = array ?: hist->stat;
 
-  if (array)
-    {
-      if (current_lrvalue == e) current_lrvalue = array;
-      if (current_lvalue == e) current_lvalue = array;
-      functioncall_traversing_visitor::visit_arrayindex (e);
-    }
-  else // if (hist)
-    {
-      if (current_lrvalue == e) current_lrvalue = hist->stat;
-      if (current_lvalue == e) current_lvalue = hist->stat;
-      functioncall_traversing_visitor::visit_arrayindex (e);
-    }
+  if (current_lrvalue == e) current_lrvalue = value;
+  if (current_lvalue == e) current_lvalue = value;
+  functioncall_traversing_visitor::visit_arrayindex (e);
 
   current_lrvalue = last_lrvalue;
   current_lvalue = last_lvalue;
@@ -2255,13 +2197,7 @@ varuse_collecting_visitor::visit_foreach_loop (foreach_loop* s)
   // NB: we duplicate so don't bother call
   // functioncall_traversing_visitor::visit_foreach_loop (s);
 
-  symbol *array = NULL;
-  hist_op *hist = NULL;
-  classify_indexable (s->base, array, hist);
-  if (array)
-    array->visit(this);
-  else
-    hist->visit(this);
+  s->base->visit(this);
 
   // If the collection is sorted, imply a "write" access to the
   // array in addition to the "read" one already noted above.
@@ -2893,26 +2829,6 @@ update_visitor::visit_hist_op (hist_op* e)
   provide (e);
 }
 
-template <> indexable*
-update_visitor::require <indexable> (indexable* src, bool clearok)
-{
-  indexable *dst = NULL;
-  if (src != NULL)
-    {
-      symbol *array_src=NULL;
-      hist_op *hist_src=NULL;
-
-      classify_indexable(src, array_src, hist_src);
-
-      if (array_src)
-        dst = require (array_src);
-      else
-        dst = require (hist_src);
-      assert(clearok || dst);
-    }
-  return dst;
-}
-
 
 // ------------------------------------------------------------------------
 
